@@ -18,9 +18,9 @@
 import gtk
 import gnome
 import gobject
+import re
 import soundcard
 import cfg
-import widgets
 
 #  Prefixes used in this module:
 #  t  pack into a table, the first five parameters are table, x1, x2, y1, y2
@@ -178,7 +178,7 @@ class FlashBar(gtk.Frame):
         #FIXME different gtk themes can make these values wrong
         self.set_size_request(-1, 40)
         self.__stack = []
-        self.__label = widgets.HarmonicProgressionLabel('')
+        self.__label = HarmonicProgressionLabel('')
         self.add(self.__label)
         self.__timeout = None
         # The allocated size
@@ -322,4 +322,72 @@ def create_stock_menu_item(stock, txt, callback, ag, accel_key, accel_mod):
     item.connect('activate', callback)
     return item
 
+class HarmonicProgressionLabel(gtk.HBox):
+    """
+    This class can parse strings like I-(6,4)V(5,3)-I and can be used
+    as button labels.
+    """
+    def __init__(self, str, align=''):
+        gtk.HBox.__init__(self)
+        self.show()
+        self.set_text(str, align)
+    def set_text(self, str, align=''):
+        for o in self.get_children():
+            o.destroy()
+        self.m_str = str
+        if align == 'center':
+            self.pack_start(gtk.HBox())
+        while self.m_str:
+            T, A, B = self.get_next_token()
+            if T == 'big' or T == 'err':
+                self.bigchar(A)
+            elif T == 'two':
+                self.twoline(A, B)
+            else:
+                assert T == 'one'
+                self.oneline(A)
+        if align == 'center':
+            self.pack_start(gtk.HBox())
+        self.show_all()
+    def get_next_token(self):
+        m_re1 = re.compile("([^\(]+)")
+        m_re2 = re.compile("\((\w*),\s*(\w*)\)")
+        m_re3 = re.compile("\((\w*)\)")
+        m1 = m_re1.match(self.m_str)
+        m2 = m_re2.match(self.m_str)
+        m3 = m_re3.match(self.m_str)
+        if m1:
+            self.m_str = self.m_str[len(m1.group()):]
+            return "big", m1.groups()[0], None
+        if m2:
+            self.m_str = self.m_str[len(m2.group()):]
+            return "two", m2.groups()[0], m2.groups()[1]
+        if m3:
+            self.m_str = self.m_str[len(m3.group()):]
+            return "one", m3.groups()[0], None
+        return "err"
+    def twoline(self, A, B):
+        vbox = gtk.VBox()
+        t1 = gtk.Label(A)
+        t1.set_name("ProgressionLabelNumber")
+        t1.show();vbox.pack_start(t1);
+        t1.set_alignment(0, 0);
+        t2 = gtk.Label(B)
+        t2.set_name("ProgressionLabelNumber")
+        t2.show();vbox.pack_start(t2);
+        t2.set_alignment(0, 0);
+        self.pack_start(vbox, False)
+    def oneline(self, A):
+        vbox = gtk.VBox()
+        t = gtk.Label(A)
+        t.set_name("ProgressionLabelNumber")
+        t.show();vbox.pack_start(t);
+        t.set_alignment(0, 0);
+        self.pack_start(vbox, False)
+    def bigchar(self, A):
+        t1 = gtk.Label(A)
+        t1.set_name("ProgressionNameLabel")
+        t1.show()
+        self.pack_start(t1, False)
+
 
