@@ -24,18 +24,17 @@ if __name__ == '__main__':
     import gtkhtml2
 
 
+import gobject
 import sys
 import gtk, pango
+import runtime
 
 #gtkhtml2.gtk_html_context_get().set_property('debug-painting', 1)
 
 import htmllib, formatter
 import i18n
 
-import re, os
-import pyscript
-import configureoutput
-import cfg
+import os
 
 class _HtmlWidgetCommon(object):
     def __init__(self):
@@ -353,11 +352,9 @@ class TextViewHtmlWidget(gtk.ScrolledWindow, _HtmlWidgetCommon):
         self.g_tw.set_wrap_mode(gtk.WRAP_WORD)
         self.add(self.g_tw)
         self.idx = 0
-        self.m_pyscript = pyscript.PyScript()
         self.grab_focus = self.g_tw.grab_focus
     def source(self, txt):
         self.writer.m_link_list = []
-        txt = self.m_pyscript.evaluate(txt)
         buf = self.g_tw.get_buffer()
         it_a = buf.get_start_iter()
         it_b = buf.get_end_iter()
@@ -406,6 +403,11 @@ class GtkHtml2Widget(gtk.ScrolledWindow, _HtmlWidgetCommon):
         self.g_document.open_stream("text/html")
         self.g_document.write_stream(html)
         self.g_document.close_stream()
+        #FIXME This is just a workaround. The problem is that when clicking on a link
+        # the next page is scrolled so that the first link on a page is visible.
+        def f1():
+            self.get_vadjustment().set_value(0.0)
+        gobject.idle_add(f1)
 
 s = """<html>
 <head>
@@ -663,7 +665,7 @@ def _test():
     w = TestWin()
     gtk.mainloop()
 
-if configureoutput.WITH_GTKHTML:
+if runtime.has_gtkhtml2():
     import gtkhtml2
     HtmlWidget = GtkHtml2Widget
 else:
