@@ -24,6 +24,7 @@ import statistics, statisticsviewer
 import lessonfile
 import dataparser
 import soundcard
+import osutils
 
 
 class Teacher(abstract.LessonbasedTeacher):
@@ -354,6 +355,9 @@ class Gui(abstract.LessonbasedGui):
             self.g_win.display_question_music_error_message(
                 self.m_t.m_P._idx, self.get_string('lessonfile'),
                 e, __file__)
+        except osutils.RunExternalFailed, e:
+            exception_cleanup()
+            self.g_win.display_exception_message(e, __file__)
     def repeat_question(self, *w):
         self.g_bb.grab_focus_first_button()
         self.m_t.m_P.play_question()
@@ -409,6 +413,7 @@ class Gui(abstract.LessonbasedGui):
         self.g_show.hide()
         self.g_repeat_arpeggio.hide()
         self.g_repeat_slowly.hide()
+        self.g_cancel_test.show()
     def exit_test_mode(self):
         self.set_bool('new_question_automatically', self.m_saved_q_auto)
         self.set_float('seconds_before_new_question', self.m_saved_s_new)
