@@ -15,24 +15,15 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-__exercise_data__ = {
-    'exercise_name': 'identify-bpm',
-    'stock-icon': 'solfege-identify-bpm',
-    'menupath': '/Menubar/FileMenu/MiscMenu',
-    'name': _("_Bpm"),
-}
-import exercise_setup
-exercise_setup.register_exercise(__exercise_data__)
-
 import gnome, gtk, gobject
 import soundcard
 import abstract, const, statistics, statisticsviewer, gu, random
+import lessonfile
 
-
-class Teacher(abstract.Teacher):
-    exercise_data = __exercise_data__
-    def __init__(self, exname, app, config):
-        abstract.Teacher.__init__(self, exname, app, config)
+class Teacher(abstract.LessonbasedTeacher):
+    def __init__(self, exname, app):
+        abstract.LessonbasedTeacher.__init__(self, exname, app)
+        self.lessonfileclass = lessonfile.IdByNameLessonfile
         # 60, 84, 100, 120
         self.m_ped = [60, 120, 40, 168, 208, 80, 96, 139, 48, 69, 108, 192, 152,
                       88, 63, 100, 116, 176, 200, 52, 76, 126, 160,
@@ -41,7 +32,7 @@ class Teacher(abstract.Teacher):
                        72, 76, 80, 84, 88, 92, 96, 100, 104,    # 9 - 17
                        108, 112, 116, 120, 126, 132, 138, 144,  # 18 - 25
                        152, 160, 168, 176, 184, 192, 200, 208]  # 26 - 33
-        #self.m_statistics = statistics.Statistics(self)
+        self.m_statistics = statistics.LessonStatistics(self)
         self.m_timeout_tag = None
         self.m_question = None
         self.m_practise_these = {}
