@@ -21,20 +21,19 @@ import pprint
 import copy
 import sys
 import os
+import cfg
+import gu
 
 class LearningTree:
     def __init__(self, lessonfile_manager):
         self.lessonfile_manager = lessonfile_manager
     def load(self, filename):
         self.m_visibility = 0
+        g = {'_': lambda s: s}
         try:
-            self.m_topics, self.m_deps = eval(file(filename, 'r').read())
+            self.m_topics, self.m_deps = eval(file(filename, 'r').read(), g)
         except ValueError:
-            self.m_topics, self.m_deps, self.m_visibility = eval(file(filename, 'r').read())
-        except IOError, e:
-            print >> sys.stderr, "Error while loading learning tree", e
-            self.m_topics = []
-            self.m_deps = {}
+            self.m_topics, self.m_deps, self.m_visibility = eval(file(filename, 'r').read(), g)
         junk_id = []
         for topic in self.m_topics:
             for j in topic['lessons']:
@@ -51,7 +50,16 @@ class LearningTree:
         self.calculate_visibilities()
     def save(self, filename):
         ofile = file(filename, 'w')
-        ofile.write(pprint.pformat([self.m_topics, self.m_deps, self.m_visibility]))
+        print >> ofile, "[ # toplevel list"
+        print >> ofile, "  [ # start of topics list"
+        for topic in self.m_topics:
+            print >> ofile, "    {'name': _('%s'), " % topic['name']
+            print >> ofile, "     'lessons': ["
+            for lesson_id in topic['lessons']:
+                print >> ofile, "          '%s'," % lesson_id
+            print >> ofile, "     ]},"
+        print >> ofile, "  ], # end of topics list"
+        ofile.write("%s,\n%s]" % (pprint.pformat(self.m_deps), self.m_visibility))
         ofile.close()
     def new_topic(self, topicname):
         assert topicname not in self.m_topics
@@ -88,18 +96,6 @@ class LearningTree:
             return False
     def delete_lesson(self, topic_idx, lesson_idx):
         del self.m_topics[topic_idx]['lessons'][lesson_idx]
-    def move_lesson_up(self, topic_idx, lesson_idx):
-        assert lesson_idx > 0
-        self.m_topics[topic_idx]['lessons'][lesson_idx], \
-            self.m_topics[topic_idx]['lessons'][lesson_idx-1] = \
-            self.m_topics[topic_idx]['lessons'][lesson_idx-1], \
-            self.m_topics[topic_idx]['lessons'][lesson_idx]
-    def move_lesson_down(self, topic_idx, lesson_idx):
-        assert lesson_idx < len(self.m_topics[topic_idx]['lessons']) - 1
-        self.m_topics[topic_idx]['lessons'][lesson_idx], \
-            self.m_topics[topic_idx]['lessons'][lesson_idx+1] = \
-            self.m_topics[topic_idx]['lessons'][lesson_idx+1], \
-            self.m_topics[topic_idx]['lessons'][lesson_idx]
     def add_dependency(self, topic_idx, lesson_idx, dep_id):
         lesson_id = self.m_topics[topic_idx]['lessons'][lesson_idx]
         assert dep_id not in self.m_deps[lesson_id]
@@ -119,6 +115,16 @@ class LearningTree:
         """
         for i in self.m_topics[n]['lessons']:
             yield i
+    def iterate_topics_for_id(self, lesson_id):
+        """
+        Yield a tuple (INT, STR) for each topic lesson_id is
+        a member of, where
+        INT -- the number of the topic
+        STR -- the title of the topic
+        """
+        for n, topic in enumerate(self.m_topics):
+            if lesson_id[0] in topic['lessons']:
+                yield (n, topic['name'])
     def iterate_deps_for_id(self, lesson_id):
         """
         Iterate all the direct dependencies for lesson_id.
@@ -241,11 +247,13 @@ class LearningTree:
         pprint.pprint(self.m_deps)
 
 
-class SelectLessonFileDialog(gtk.Dialog):
-    def __init__(self, app, tree, lesson_id_list):
+class LessonFileDialogCommon(gtk.Dialog):
+    def __init__(self, app, tree, cur_topic_idx):
         gtk.Dialog.__init__(self, _("Select lesson file"), self, buttons=
             (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,))
+        self.m_app = app
         self.m_tree = tree
+        self.m_cur_topic_idx = cur_topic_idx
         self.g_ok_button = self.add_button("gtk-ok", gtk.RESPONSE_OK)
         self.g_ok_button.set_sensitive(False)
         self.set_default_size(800, 500)
@@ -254,17 +262,14 @@ class SelectLessonFileDialog(gtk.Dialog):
         self.g_liststore = gtk.ListStore(gobject.TYPE_STRING,
             gobject.TYPE_STRING, gobject.TYPE_STRING, gobject.TYPE_INT,
             gobject.TYPE_STRING)
-        for lesson_id in lesson_id_list:
-            d = app.lessonfile_manager.m_uiddb[lesson_id]
-            self.g_liststore.append((d['header']['title'], d['filename'], lesson_id, self.m_tree.get_use_count(lesson_id), app.lessonfile_manager.get(lesson_id, 'module')))
+        self.update_filelist()
         self.g_treeview = gtk.TreeView(self.g_liststore)
         self.g_treeview.connect('cursor-changed', self.on_cursor_changed)
         renderer = gtk.CellRendererText()
         column = gtk.TreeViewColumn(_("Used"), renderer, text=3)
+        column.set_sort_column_id(3)
         column.set_resizable(True)
         self.g_treeview.append_column(column)
-        def sort_func1(store, itera, iterb):
-            return cmp(store.get(itera, 0)[0], store.get(iterb, 1)[0])
         column = gtk.TreeViewColumn(_("Title"), renderer, text=0)
         column.set_resizable(True)
         column.set_sort_column_id(0)
@@ -286,15 +291,19 @@ class SelectLessonFileDialog(gtk.Dialog):
         hbox.pack_start(gtk.Label(_("Module:")), False)
         self.g_module = gtk.Label()
         hbox.pack_start(self.g_module, False)
+        #
+        hbox = gtk.HBox()
+        hbox.set_spacing(8)
+        hbox.set_border_width(12)
+        self.vbox.pack_start(hbox, False)
+        label = gtk.Label(_("Used by topic(s):"))
+        label.set_alignment(0.0, 0.0)
+        hbox.pack_start(gtk.Label(_("Used by topic(s):")), False)
+        self.g_topics_vbox = gtk.VBox()
+        hbox.pack_start(self.g_topics_vbox, False)
+        #
         self.show_all()
         self.g_treeview.connect('row-activated', self.on_row_activated)
-    def on_row_activated(self, treeview, row, treeviewcolumn):
-        self.response(gtk.RESPONSE_OK)
-    def on_cursor_changed(self, treeview):
-        i = self.g_treeview.get_cursor()[0][0]
-        iter = self.g_liststore.iter_nth_child(None, i)
-        self.g_module.set_text(self.g_liststore.get(iter, 4)[0])
-        self.g_ok_button.set_sensitive(True)
     def get_selected_lesson_id(self):
         """
         Returns None if no lesson file is selected.
@@ -304,6 +313,54 @@ class SelectLessonFileDialog(gtk.Dialog):
             i = i[0]
             iter = self.g_liststore.iter_nth_child(None, i)
             return self.g_liststore.get(iter, 2)[0]
+    def on_row_activated(self, treeview, row, treeviewcolumn):
+        self.response(gtk.RESPONSE_OK)
+    def on_cursor_changed(self, treeview):
+        i = self.g_treeview.get_cursor()[0][0]
+        iter = self.g_liststore.iter_nth_child(None, i)
+        self.g_module.set_text(self.g_liststore.get(iter, 4)[0])
+        self.g_topics_vbox.foreach(lambda w: w.destroy())
+        for n, s in self.m_tree.iterate_topics_for_id(
+                self.g_liststore.get(iter, 2)):
+            label = gtk.Label(s)
+            label.set_alignment(0.0, 0.5)
+            label.show()
+            self.g_topics_vbox.pack_start(label)
+        self.g_ok_button.set_sensitive(True)
+
+
+class SelectLessonFileDialog(LessonFileDialogCommon):
+    def __init__(self, app, tree, cur_topic_idx):
+        LessonFileDialogCommon.__init__(self, app, tree, cur_topic_idx)
+        g_show_used = gu.nCheckButton('SelectLessonFileDialog',
+            'show_used_files', _("_Show files used in other topics"),
+            False, self.update_filelist)
+        self.vbox.pack_start(g_show_used, False)
+        self.vbox.reorder_child(g_show_used, 0)
+    def update_filelist(self, *v):
+        self.g_liststore.clear()
+        if cfg.get_bool('SelectLessonFileDialog/show_used_files'):
+            lesson_id_list = [i for i in list(self.m_app.lessonfile_manager.iterate_lesson_ids()) if i not in self.m_tree.m_topics[self.m_cur_topic_idx]['lessons']]
+        else:
+            lesson_id_list = [i for i in list(self.m_app.lessonfile_manager.iterate_lesson_ids()) if self.m_tree.get_use_count(i) == 0]
+        for lesson_id in lesson_id_list:
+            d = self.m_app.lessonfile_manager.m_uiddb[lesson_id]
+            self.g_liststore.append((d['header']['title'], d['filename'], lesson_id, self.m_tree.get_use_count(lesson_id), self.m_app.lessonfile_manager.get(lesson_id, 'module')))
+
+
+class DepsLessonFileDialog(LessonFileDialogCommon):
+    def __init__(self, app, tree, cur_topic_idx, i):
+        """
+        i is the index from the lesson in the topic in the tree.
+        """
+        self.m_idx = i
+        LessonFileDialogCommon.__init__(self, app, tree, cur_topic_idx)
+        #self.update_filelist(i)
+    def update_filelist(self):
+        self.g_liststore.clear()
+        for lesson_id in list(self.m_tree.iterate_possible_deps_for(self.m_cur_topic_idx, self.m_idx)):
+            d = self.m_app.lessonfile_manager.m_uiddb[lesson_id]
+            self.g_liststore.append((d['header']['title'], d['filename'], lesson_id, self.m_tree.get_use_count(lesson_id), self.m_app.lessonfile_manager.get(lesson_id, 'module')))
 
 
 class Window(gtk.Window):
@@ -318,7 +375,7 @@ class Window(gtk.Window):
         g = gtk.Button(_("Save"))
         g.connect('clicked', self.on_save)
         hbox.pack_start(g, False)
-        g = gtk.Button(_("Save as"))
+        g = gtk.Button(_("Save as..."))
         g.connect('clicked', self.on_save_as)
         hbox.pack_start(g, False)
         g = gtk.Button(_("Close"))
@@ -476,11 +533,10 @@ class Window(gtk.Window):
     def on_delete_event(self, *v):
         self.close_window()
     def close_window(self, *v):
-        if not (self.m_tree.m_topics == self._m_orig_topics
-                and self.m_tree.m_deps == self._m_orig_deps
-                and self.m_tree.m_visibility == self._m_orig_visibility):
+        if self.tree_has_changes():
             if self.m_app.m_ui.dialog_yesno(_("Save changes?")):
                 self.on_save()
+        self.m_app.m_ui.g_learning_tree_editor = None
         self.destroy()
     def on_topic_title_edited(self, renderer, path, txt):
         iter = self.g_topics_liststore.iter_nth_child(None, int(path))
@@ -572,6 +628,13 @@ class Window(gtk.Window):
             self._m_orig_deps = copy.deepcopy(self.m_tree.m_deps)
             self._m_orig_topics = copy.deepcopy(self.m_tree.m_topics)
             self._m_orig_visibility = self.m_tree.m_visibility
+    def tree_has_changes(self):
+        """
+        Return True if the tree has unsaved changes.
+        """
+        return not (self.m_tree.m_topics == self._m_orig_topics
+                and self.m_tree.m_deps == self._m_orig_deps
+                and self.m_tree.m_visibility == self._m_orig_visibility)
     def on_save_as(self, *v):
         dialog = gtk.FileChooserDialog(_("Save as..."), self,
                                    gtk.FILE_CHOOSER_ACTION_SAVE,
@@ -593,7 +656,7 @@ class Window(gtk.Window):
         vbox = gtk.VBox()
         dlg.vbox.pack_start(vbox)
         vbox.set_border_width(18)
-        self.g_heading = gtk.Label(_("<b>Create new topic</b>"))
+        self.g_heading = gtk.Label("<b>%s</b>" % _("Create new topic"))
         self.g_heading.set_alignment(0.0, 0.5)
         vbox.pack_start(self.g_heading, False)
         hbox = gtk.HBox()
@@ -635,6 +698,8 @@ class Window(gtk.Window):
                 itera = self.g_topics_liststore.iter_nth_child(None, i)
                 iterb = self.g_topics_liststore.iter_nth_child(None, i - 1)
                 self.g_topics_liststore.swap(itera, iterb)
+                self.g_topics.scroll_to_cell(
+                    self.g_topics_liststore.get_string_from_iter(itera))
     def on_move_topic_down(self, *v):
         i = self.g_topics.get_cursor()[0]
         if i:
@@ -643,6 +708,8 @@ class Window(gtk.Window):
             iterb = self.g_topics_liststore.iter_next(itera)
             if iterb:
                 self.g_topics_liststore.swap(itera, iterb)
+                self.g_topics.scroll_to_cell(
+                    self.g_topics_liststore.get_string_from_iter(itera))
                 self.m_tree.move_topic_down(i)
     def on_delete_dep(self, *v):
         i = self.g_deps.get_cursor()[0]
@@ -662,7 +729,7 @@ class Window(gtk.Window):
         if i:
             i = i[0]
             topic_idx = self.g_topics.get_cursor()[0][0]
-            dlg = SelectLessonFileDialog(self.m_app, self.m_tree, list(self.m_tree.iterate_possible_deps_for(topic_idx, i)))
+            dlg = DepsLessonFileDialog(self.m_app, self.m_tree, topic_idx, i)
             ret = dlg.run()
             if ret == gtk.RESPONSE_OK:
                 lesson_id = dlg.get_selected_lesson_id()
@@ -674,7 +741,7 @@ class Window(gtk.Window):
             dlg.destroy()
     def on_new_lesson(self, *v):
         topic_idx = self.g_topics.get_cursor()[0][0]
-        dlg = SelectLessonFileDialog(self.m_app, self.m_tree, [i for i in list(self.m_app.lessonfile_manager.iterate_lesson_ids()) if i not in self.m_tree.m_topics[topic_idx]['lessons']])
+        dlg = SelectLessonFileDialog(self.m_app, self.m_tree, topic_idx)
         dlg.run()
         lesson_id = dlg.get_selected_lesson_id()
         if lesson_id:
@@ -691,7 +758,7 @@ class Window(gtk.Window):
             do_del = True
             if self.m_tree.get_use_count(del_id) == 1:
                 if self.m_tree.get_dep_use_count(del_id) > 0:
-                    do_del = self.m_app.m_ui.dialog_yesno("Lessons depend on this lesson. Delete anyway? (Dependencies will be removed)")
+                    do_del = self.m_app.m_ui.dialog_yesno(_("Lessons depend on this lesson. Delete anyway, and update the dependency list for lessons that depend on this lesson?"))
                     self.m_tree.remove_all_deps_of(del_id)
             if do_del:
                 self.m_tree.delete_lesson(topic_idx, i)
