@@ -956,7 +956,7 @@ class LessonFileManager:
         FIXME: the function name is not very good.
         Return data used to fix things when we have a lesson_id crash.
         """
-        return [{'filename': d['filename'], 'lessoncollection': d['lessoncollection'], 'lessonfile': d['lessonfile'], 'timestr': time.strftime(locale.nl_langinfo(locale.D_T_FMT), time.localtime(d['mtime'])), 'mtime': d['mtime']} for d in self.m_uiddb[lesson_id]]
+        return [{'filename': d['filename'], 'lessoncollection': d['lessoncollection'], 'lessonfile': d['lessonfile'], 'timestr': time.strftime('%c', time.localtime(d['mtime'])), 'mtime': d['mtime']} for d in self.m_uiddb[lesson_id]]
     def get_complete_filename(self, catalog, fn):
         """
         Return the absolute path to a file in a given collection
