@@ -16,7 +16,6 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 import gtk, gnome.ui, gobject
-import widgets
 import abstract, const, gu, mpd, soundcard
 import random, os
 import cfg
@@ -204,7 +203,7 @@ class RhythmViewer(gtk.Frame):
         "return the number of rhythm elements currently viewed"
         return len(self.m_data)
     def sad_face(self):
-        l = widgets.HarmonicProgressionLabel(_("Wrong"))
+        l = gu.HarmonicProgressionLabel(_("Wrong"))
         l.show()
         self.g_box.pack_start(l, False)
         self.g_face = gtk.EventBox()
@@ -216,7 +215,7 @@ class RhythmViewer(gtk.Frame):
         self.g_face.add(im)
         self.g_box.pack_start(self.g_face, False)
     def happy_face(self):
-        l = widgets.HarmonicProgressionLabel(_("Correct"))
+        l = gu.HarmonicProgressionLabel(_("Correct"))
         l.show()
         self.g_box.pack_start(l, False)
         self.g_face = gtk.EventBox()
@@ -285,7 +284,7 @@ class Gui(abstract.Gui):
         ##############
         # config_box #
         ##############
-        frame = gtk.Frame(_("Rhythms to use in question"))
+        self.g_element_frame = frame = gtk.Frame(_("Rhythms to use in question"))
         self.config_box.pack_start(frame, False)
         vbox = gtk.VBox()
         vbox.set_border_width(gnome.ui.PAD_SMALL)
@@ -406,15 +405,27 @@ class Gui(abstract.Gui):
         self.m_t.play_question()
         self.g_first_rhythm_button.grab_focus()
     def on_start_practise(self):
-        for n in range(22):
-            self.m_t.set_bool('rhythm_element_%i' % n, 
+        if self.m_t.m_P.header.rhythm_elements:
+            for n in range(22):
+                self.m_t.set_bool('rhythm_element_%i' % n, 
                               n in self.m_t.m_P.header.rhythm_elements)
-        self.m_t.set_int('num_beats', self.m_t.m_P.header.num_beats)
-        self.m_t.set_int('bpm', self.m_t.m_P.header.bpm)
-        self.m_t.set_int('countin_perc', self.m_t.m_P.header.countin_perc)
-        self.m_t.set_int('rhythm_perc', self.m_t.m_P.header.rhythm_perc)
-        self.m_t.set_int('count_in', self.m_t.m_P.header.count_in)
-        self.m_t.set_bool('not_start_with_rest', self.m_t.m_P.header.not_start_with_rest)
+        self.m_t.m_custom_mode = bool(
+            not self.m_t.m_P.header.rhythm_elements)
+        if self.m_t.m_custom_mode:
+            self.g_element_frame.show()
+        else:
+            self.g_element_frame.hide()
+        if 'lesson_heading' in self.m_t.m_P.header:
+            self.set_lesson_heading(self.m_t.m_P.header.lesson_heading)
+        else:
+            self.set_lesson_heading(_("Identify the rhythm"))
+        for n in ('bpm',
+                  'countin_perc', 'count_in',
+                  'rhythm_perc', 'num_beats'):
+            if n in self.m_t.m_P.header:
+                self.m_t.set_int(n, self.m_t.m_P.header[n])
+        if 'not_start_with_rest' in self.m_t.m_P.header:
+            self.m_t.set_bool('not_start_with_rest', self.m_t.m_P.header.not_start_with_rest)
         self.g_rhythm_viewer.flash(_("Click 'New' to begin."))
         self.g_new.grab_focus()
     def on_end_practise(self):
