@@ -19,13 +19,10 @@
 # Remember to increase statistics_version and add code to handle
 # old versions if incompatabilities are added.
 
-import sys
 if __name__ == '__main__':
-    sys.path.append("src")
-    sys.path.append(".")
-    import i18n
-    i18n.setup_srcdir()
+    import testsetup
 
+import sys
 import time, os, pickle
 import shutil
 import utils
